Module station

Module station 

Station and support classes.

This class and its support classes implement information about stations and station stops. This includes where a station is along the line (its scale mile), what storage tracks it has, and what trains are being stored on the storage tracks during what times. Stations are places where trains stop or just important junctions or mile post locations that trains pass by – they might only be used for time keeping checks. Note: the classification tracks at a yard are not storage tracks. Nor are RIP or service or other special purpose tracks. Storage tracks are only for storing whole, complete trains (they might be without engines).

Structs

Occupied
This class records a train sitting on a storage track during a specified time frame. The train number (symbol) might change when the train leaves the storage track.
Station
The Station class implements a station. Stations are not specifically passenger stations, but are any place where trains stop or meet or might just be important mile post locations used for time keeping checks. They also can be just sidings.
StorageTrack
The StorageTrack class implements a storage track.
TimeRange
The TimeRange class implements a range of times.

Enums

OccupiedParseError
StationParseError
StorageTrackParseError
TimeRangeParseError

Type Aliases

OccupiedMap
The Occupied Map type, ordered by time ranges.
StationVector
Station Vector.
StorageTrackMap
Storage track map. Indexed by name.